function love.load()
    canvas = love.graphics.newCanvas()
    shader = love.graphics.newPixelEffect [[
        extern number time = 0.0;
        extern number x = 0.25;
        extern number y = -0.25;
        extern number size = 32.0;
        extern number strength = 8.0;
        extern vec2 res = vec2(512.0, 512.0);
        uniform sampler2D tex0;
        vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
        {
            float tmp = cos(clamp(sqrt(pow((texture_coords.x - x) * size - size / 2.0, 2.0) + pow((texture_coords.y - y) * size - size / 2.0, 2.0)) - time * 16.0, -3.1415, 3.1415));
            vec2 uv         = vec2(
                texture_coords.x - tmp * strength / 1024.0,
                texture_coords.y - tmp * strength / 1024.0
            );
         return vec4(texture2D(tex0,uv));
        }
    ]]
    tex = love.graphics.newImage('tex.png')
    t = 0
    size = 32
    strength = 8
    love.keyboard.setKeyRepeat(0.1, 0.05)
end

function love.update(dt)
    t = t + dt
end

function love.draw()
    love.graphics.setCanvas(canvas)
    love.graphics.setPixelEffect(shader)

    --first ripple, at mouse cursor
    shader:send('x', love.mouse.getX()/800 - .5)
    shader:send('y', love.mouse.getY()/600 - .5)
    shader:send('time', t)
    love.graphics.draw(tex, 0, 0)

    --second ripple, top left corner
    shader:send('x', -.5)
    shader:send('y', .5)
    shader:send('time', t - .5)
    love.graphics.draw(canvas, 0, 0)

    --third ripple, bottom right corner
    shader:send('x', .5)
    shader:send('y', -.5)
    shader:send('time', t - 1)
    love.graphics.draw(canvas, 0, 0)

    
    love.graphics.setPixelEffect()
    love.graphics.setCanvas()
    love.graphics.draw(canvas, 0, 0)
    
    love.graphics.setColor(0, 0, 0)
    love.graphics.rectangle('fill', 0, 0, 128, 64)
    love.graphics.setColor(255, 255, 255)
    love.graphics.print('size = ' .. size .. '\nstrength = ' .. strength .. '\nFPS = ' .. love.timer.getFPS() .. '\n' .. 'Press t to restart', 0, 0)
end

function love.keypressed(key, unicode)
    if key == 'escape' then
        love.event.quit()
    end
    if key == 'up' then
        size = size + 8
        shader:send('size', size)
    end
    if key == 'down' then
        size = size - 8
        shader:send('size', size)
    end
    if key == 'left' then
        strength = strength - 1
        shader:send('strength', strength)
    end
    if key == 'right' then
        strength = strength + 1
        shader:send('strength', strength)
    end
    if key == 't' then
        t = 0
    end
end
